/********************************************************/  
  /* AD PREVODNIK - MERENI NAPETI A PROUDU */
  /* MCP3202 12-bit 2 channel ADC */
/********************************************************/
/* Hlavni funkce pro komunikaci s ADC prevodnikem
    Vstupy  - PSU_channel = 1 nebo 2
            - ADC_channel = 'U' nebo 'I' 
    Vystup  - float hodnota vystupniho napeti (0-30.00 [V]) nebo proudu (0 - 1.000 [A])
*/
float ADC_read(uint8_t PSU_channel, char ADC_channel) {
  word AD_value;                                               
  uint8_t ADch = 0;
  if(ADC_channel == 'U')  
    ADch = 1;                                                     //Vystupni napeti je pripojeno na kanal 1 ADC
  else                    
    ADch = 0;                                                     //Vystupni proud je pripojen na kanal 0 ADC
  if(PSU_channel == 1)
    AD_value = ADC_transfer(CS_ADC1, ADch);                       //PSU Channel 1
  else                    
    AD_value = ADC_transfer(CS_ADC2, ADch);                       //PSU Channel 2
  //Kompenzace hodnoty kalibracnimi konstantami
  AD_value = AD_komepnzace(AD_value, PSU_channel, ADC_channel);  
  //Prevedeni hodnoty z ADC na vystupni napeti/proud
  float result = ADC_convert2actual(AD_value, ADch);
  //lcd.setCursor(0,0);
  //lcd.print(AD_value);
  return result;
}

//////////////////////////////////////////////////////////////////////////////
/* Pocatecni nastaveni pinu */
void ADC_init(void) {
  pinMode(CS_ADC1, OUTPUT);
  digitalWrite(CS_ADC1, HIGH);
  pinMode(CS_ADC2, OUTPUT);
  digitalWrite(CS_ADC2, HIGH);
  return;
}

//////////////////////////////////////////////////////////////////////////////
/* Funkce pro prenos dat s ADC MCP3202 - celkem 24 bitu na ziskani 12bit hodnoty word_result */
word ADC_transfer(uint8_t CSpin, uint8_t ADC_channel) {
  word word_result = 0;                                           //16 bitove slovo
  if(CSpin == CS_ADC2) {                                          //Pokud chci komunikovat s Kanalem 2, povol MISO komunikaci (na gate passgate NMOSu privedu log. 1)
    digitalWrite(CH2_EN_SP16, HIGH);
  }
  digitalWrite(CSpin, LOW);
  //1 Datovy prenos = //0000 000ST + SE CH MSB x xxxx + xxxx xxxx
  //STart bit | Signle Ended = 1| Channel = 0/1 | MSB first = 1 |
  SPI.transfer(0x01);                                             //Prvnich 8 bitu - 7 bitu = leading zeroes, 1 bit = start bit
  //16 bitu = 3 bity nastaveni ADC, null bit, 12 bitu ADC data (prenaseno behem (x xxxx xxxx xxxx) faze (don't care bity))
  if(ADC_channel == 0 || ADC_channel == 'I')
    word_result = SPI.transfer16(0xA000);                         //ADC Channel 0 - 101x xxxx xxxx xxxx 
  else if(ADC_channel == 1 || ADC_channel == 'U')
    word_result = SPI.transfer16(0xE000);                         //ADC Channel 1 - 111x xxxx xxxx xxxx
  digitalWrite(CSpin, HIGH);
  digitalWrite(CH2_EN_SP16, LOW);
  return word_result;
}

//////////////////////////////////////////////////////////////////////////////
/* Prevodni funkce z mereneho napeti na vystupni napeti / vystupni proud kanalu PSU */
float ADC_convert2actual (word AD_value, uint8_t ADC_channel) {
  float result = 0;
  if(ADC_channel == 0)  
    result = AD_value*A_per_bit;                                  //ADC Channel 0 - Vystupni proud  0-1 A
  else                  
    result = AD_value*V_per_bit;                                  //ADC Channel 1 - Vystupni napeti 0-30 V
  return result;
}
//////////////////////////////////////////////////////////////////////////////
/* Zmereni novych hodnot vystupniho napeti a proudu */
void ADC_measure_values(void) {
  Ch1.Vread = ADC_read(1, 'U');
  Ch1.Iread = ADC_read(1, 'I');
  Ch1.Iread = round(Ch1.Iread*1000)/1000.0;                       //Zaokrouhleni na jednotky mA (z rozliseni 0.25 mA)
  Ch2.Vread = ADC_read(2, 'U');   //! ! ! TBD 
  Ch2.Iread = ADC_read(2, 'I');
  Ch2.Iread = round(Ch2.Iread*1000)/1000.0;
} //End ADC_measure_values
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
/* Mereni vsech U,I a ulozeni vzorku do pameti pro prumerovani, neaktualizuje hodnotu na displeji */
void perform_measurement(void) {
  if(currentMillis - lastMeasurement >= MEASUREMENT_PERIOD) {
    lastMeasurement = currentMillis;
    //float last_samples[4][AVG_SAMPLES] = Array poslednich AVG_SAMPLES vzorku, last_samples[U1, I1, U2, I2][x-AVG_SAMPLES,....x-2 ,x-1 ,x(nejnovejsi sample)]
    //Posun vzorku v arrayi smerem k indexu [x][0], zahozeni nejstarsiho vzorku co je na indexu [x][0]
    for(int j=0; j<4; j++) {                          //Posun pro vsechny napeti i proudy
      for(int i=0; i<AVG_SAMPLES-1; i++) {
          last_samples[j][i] = last_samples[j][i+1];  
      }
    }
    //Nasunuti nejnovejsiho vzorku do arraye na index [x][AVG_SAMPLES]
    last_samples[0][AVG_SAMPLES-1] = ADC_read(1, 'U');
    last_samples[1][AVG_SAMPLES-1] = ADC_read(1, 'I');
    last_samples[2][AVG_SAMPLES-1] = ADC_read(2, 'U');
    last_samples[3][AVG_SAMPLES-1] = ADC_read(2, 'I');  
  }
  return;
} //End perform_measurement
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
/* Zprumerovani poslednich AVG_SAMPLES vzorku v pameti, ulozeni do globalnich promennych spojenych s jednotlivymi kanaly */
/* Nasleduje funkce ktera zobrazi prumernou hodnotu na LCD */
void get_average_values(void) {
  for(int j=0; j<4; j++){                           //Prumerovani Ch1 U, Ch1 I, Ch2 U, Ch I
    float temp = 0;
    for(int i=0; i<AVG_SAMPLES; i++){
      temp = temp + last_samples[j][i];             //Secteni vsech vzorku
    }
    if(AVG_SAMPLES != 0)
      temp = temp/AVG_SAMPLES;                      //Podeleni poctem vzorku - aritmeticky prumer
    switch(j) {
      case 0: Ch1.Vread = round(temp*100)/100.0;    //Zaokrouhleni na desitky mV 
      case 1: Ch1.Iread = round(temp*1000)/1000.0;  //Zaokrouhleni na jednotky mA
      case 2: Ch2.Vread = round(temp*100)/100.0;    //Zaokrouhleni na desitky mV 
      case 3: Ch2.Iread = round(temp*1000)/1000.0;  //Zaokrouhleni na jednotky mA
      default: break;
    }
  }
} //End get_average_values
//////////////////////////////////////////////////////////////////////////////
